@@ -3,8 +3,11 @@
 #define PQXX_HAVE_PQEXECPREPARED 1
 #define PQXX_HAVE_PQFREEMEM 1
 #define PQXX_HAVE_PQFREENOTIFY 1
-#define PQXX_HAVE_PQPROTOCOLVERSION 1
 #define PQXX_HAVE_PQPUTCOPY 1
 #define PQXX_HAVE_PQRESULTERRORFIELD 1
 #define PQXX_HAVE_PQUNESCAPEBYTEA 1
+#define PQXX_HAVE_PQPROTOCOLVERSION 1
+#define PQXX_HAVE_PQSERVERVERSION 1
+#define PQXX_HAVE_PQPREPARE 1
 #define PQXX_HAVE_PQFTABLE 1
+#define PQXX_HAVE_PQFTABLECOL 1
