@@ -7,7 +7,7 @@
  *      definitions for the pqxx::cachedresult class and support classes.
  *   pqxx::cachedresult is a lazy-fetching, transparently-cached result set
  *
- * Copyright (c) 2001-2005, Jeroen T. Vermeulen <jtv@xs4all.nl>
+ * Copyright (c) 2001-2006, Jeroen T. Vermeulen <jtv@xs4all.nl>
  *
  * See COPYING for copyright license.  If you did not receive a file called
  * COPYING with this source code, please notify the distributor of this mistake,
@@ -23,7 +23,7 @@
 #include <map>
 
 #include "pqxx/cursor.h"
-#include "pqxx/result.h"
+#include "pqxx/result"
 
 namespace pqxx
 {
@@ -126,7 +126,6 @@ public:
   bool empty() const;							//[t47]
 
 private:
-  typedef Cursor::pos pos;
 
   /// Only defined for permitted isolation levels (in this case, serializable)
   /** If you get a link or compile error saying this function is not defined,
