@@ -7,7 +7,7 @@
  *      implementation of the pqxx::connection_base abstract base class.
  *   pqxx::connection_base encapsulates a frontend to backend connection
  *
- * Copyright (c) 2001-2006, Jeroen T. Vermeulen <jtv@xs4all.nl>
+ * Copyright (c) 2001-2007, Jeroen T. Vermeulen <jtv@xs4all.nl>
  *
  * See COPYING for copyright license.  If you did not receive a file called
  * COPYING with this source code, please notify the distributor of this mistake,
@@ -92,6 +92,15 @@ static void clear_fdmask(fd_set *mask)
 }
 
 
+#ifdef PQXX_HAVE_PQENCRYPTPASSWORD
+string pqxx::encrypt_password(const string &user, const string &password)
+{
+  PQAlloc<char> p(PQencryptPassword(password.c_str(), user.c_str()));
+  return string(p.c_ptr());
+}
+#endif
+
+
 pqxx::connection_base::connection_base(connectionpolicy &pol) :
   m_Conn(0),
   m_policy(pol),
@@ -100,6 +109,7 @@ pqxx::connection_base::connection_base(connectionpolicy &pol) :
   m_Noticer(),
   m_defaultNoticeProcessor(0),
   m_Trace(0),
+  m_serverversion(0),
   m_caps(),
   m_inhibit_reactivation(false),
   m_reactivation_avoidance(),
@@ -192,6 +202,16 @@ void pqxx::connection_base::deactivate()
 }
 
 
+void pqxx::connection_base::simulate_failure()
+{
+  if (m_Conn)
+  {
+    m_Conn = m_policy.do_disconnect(m_Conn);
+    inhibit_reactivation(true);
+  }
+}
+
+
 int pqxx::connection_base::protocol_version() const throw ()
 {
 #ifdef PQXX_HAVE_PQPROTOCOLVERSION
@@ -204,11 +224,7 @@ int pqxx::connection_base::protocol_version() const throw ()
 
 int pqxx::connection_base::server_version() const throw ()
 {
-#ifdef PQXX_HAVE_PQSERVERVERSION
-  return m_Conn ? PQserverVersion(m_Conn) : 0;
-#else
-  return 0;
-#endif
+  return m_serverversion;
 }
 
 
@@ -310,7 +326,8 @@ void pqxx::connection_base::SetupState()
     // Now do the whole batch at once
     PQsendQuery(m_Conn, restore_query.str().c_str());
     result r;
-    do r = PQgetResult(m_Conn); while (r);
+    const int proto = protocol_version();
+    do r = result(PQgetResult(m_Conn), proto, "[RECONNECT]"); while (r);
   }
 
   m_Completed = true;
@@ -318,16 +335,16 @@ void pqxx::connection_base::SetupState()
 }
 
 
-void pqxx::connection_base::check_result(const result &R, const char Query[])
+void pqxx::connection_base::check_result(const result &R)
 {
   if (!is_open()) throw broken_connection();
 
-  // A shame we can't detect out-of-memory to turn this into a bad_alloc...
+  // A shame we can't quite detect out-of-memory to turn this into a bad_alloc!
   if (!R) throw runtime_error(ErrMsg());
 
   try
   {
-    R.CheckStatus(Query);
+    R.CheckStatus();
   }
   catch (const exception &e)
   {
@@ -542,15 +559,15 @@ void pqxx::connection_base::AddTrigger(pqxx::trigger *T)
   const TriggerList::iterator p = m_Triggers.find(T->name());
   const TriggerList::value_type NewVal(T->name(), T);
 
-  if (m_Conn && (p == m_Triggers.end()))
+  if (p == m_Triggers.end())
   {
     // Not listening on this event yet, start doing so.
     const string LQ("LISTEN \"" + T->name() + "\"");
-    result R( PQexec(m_Conn, LQ.c_str()) );
 
     if (is_open()) try
     {
-      check_result(R,LQ.c_str());
+      result R( PQexec(m_Conn, LQ.c_str()), protocol_version(), LQ );
+      check_result(R);
     }
     catch (const broken_connection &)
     {
@@ -582,18 +599,15 @@ void pqxx::connection_base::RemoveTrigger(pqxx::trigger *T) throw ()
 
     if (i == R.second)
     {
-      process_notice("Attempt to remove unknown trigger '" +
-		     E.first +
-		     "'");
+      process_notice("Attempt to remove unknown trigger '" + E.first + "'");
     }
     else
     {
       // Erase first; otherwise a notification for the same trigger may yet come
       // in and wreak havoc.  Thanks Dragan Milenkovic.
+      const bool gone = (m_Conn && (R.second == ++R.first));
       m_Triggers.erase(i);
-
-      if (m_Conn && (R.second == ++R.first))
-        Exec(("UNLISTEN \"" + T->name() + "\"").c_str(), 0);
+      if (gone) Exec(("UNLISTEN \"" + T->name() + "\"").c_str(), 0);
     }
   }
   catch (const exception &e)
@@ -704,16 +718,16 @@ pqxx::result pqxx::connection_base::Exec(const char Query[], int Retries)
 {
   activate();
 
-  result R( PQexec(m_Conn, Query) );
+  result R(PQexec(m_Conn, Query), protocol_version(), Query);
 
   while ((Retries > 0) && !R && !is_open())
   {
     Retries--;
     Reset();
-    if (is_open()) R = PQexec(m_Conn, Query);
+    if (is_open()) R = result(PQexec(m_Conn, Query), protocol_version(), Query);
   }
 
-  check_result(R, Query);
+  check_result(R);
 
   get_notifs();
   return R;
@@ -757,7 +771,6 @@ void pqxx::connection_base::unprepare(const PGSTD::string &name)
 }
 
 
-#ifndef PQXX_HAVE_PQEXECPREPARED
 namespace
 {
 string escape_param(const char in[],
@@ -803,7 +816,6 @@ string escape_param(const char in[],
   return in;
 }
 } // namespace
-#endif
 
 
 pqxx::prepare::internal::prepared_def &
@@ -829,22 +841,12 @@ void pqxx::connection_base::prepare_param_declare(
 }
 
 
-// TODO: Can we make this work with std::string instead of C-style?
-pqxx::result pqxx::connection_base::prepared_exec(
-	const PGSTD::string &statement,
-	const char *const params[],
-	const int paramlengths[],
-	int nparams)
+pqxx::prepare::internal::prepared_def &
+pqxx::connection_base::register_prepared(const PGSTD::string &name)
 {
   activate();
 
-  prepare::internal::prepared_def &s = find_prepared(statement);
-
-  if (nparams != int(s.parameters.size()))
-    throw logic_error("Wrong number of parameters for prepared statement " +
-	statement + ": "
-	"expected " + to_string(s.parameters.size()) + ", "
-	"received " + to_string(nparams));
+  prepare::internal::prepared_def &s = find_prepared(name);
 
   s.complete = true;
 
@@ -852,10 +854,18 @@ pqxx::result pqxx::connection_base::prepared_exec(
   if (!s.registered && supports(cap_prepared_statements))
   {
 #ifdef PQXX_HAVE_PQPREPARE
-    PQprepare(m_Conn, statement.c_str(), s.definition.c_str(), 0, 0);
-#else
+    if (protocol_version() >= 3)
+    {
+      result r(PQprepare(m_Conn, name.c_str(), s.definition.c_str(), 0, 0),
+	protocol_version(),
+	"[PREPARE " + name + "]");
+      check_result(r);
+      s.registered = true;
+      return s;
+    }
+#endif
     stringstream P;
-    P << "PREPARE \"" << statement << '"';
+    P << "PREPARE \"" << name << "\" ";
 
     if (!s.parameters.empty())
       P << '('
@@ -867,27 +877,63 @@ pqxx::result pqxx::connection_base::prepared_exec(
 
     P << " AS " << s.definition;
     Exec(P.str().c_str(), 0);
-#endif
     s.registered = true;
   }
 
-#ifdef PQXX_HAVE_PQEXECPREPARED
-  internal::scoped_array<int> binary(nparams+1);
-  for (int i=0; i<nparams; ++i)
-    binary[i] = (s.parameters[i].treatment == treat_binary);
-  binary[nparams] = 0;
+  return s;
+}
 
-  result r(PQexecPrepared(m_Conn,
+void pqxx::connection_base::prepare_now(const PGSTD::string &name)
+{
+  register_prepared(name);
+}
+
+
+// TODO: Can we make this work with std::string instead of C-style?
+pqxx::result pqxx::connection_base::prepared_exec(
+	const PGSTD::string &statement,
+	const char *const params[],
+	const int paramlengths[],
+	int nparams)
+{
+  prepare::internal::prepared_def &s = register_prepared(statement);
+
+  if (nparams != int(s.parameters.size()))
+    throw logic_error("Wrong number of parameters for prepared statement " +
+	statement + ": "
+	"expected " + to_string(s.parameters.size()) + ", "
+	"received " + to_string(nparams));
+
+  result r;
+
+  activate();
+
+  if (supports(cap_prepared_statements))
+  {
+#ifdef PQXX_HAVE_PQEXECPREPARED
+    if (protocol_version() >= 3)
+    {
+      internal::scoped_array<int> binary(nparams+1);
+      for (int i=0; i<nparams; ++i)
+        binary[i] = (s.parameters[i].treatment == treat_binary);
+      binary[nparams] = 0;
+
+      r = result(PQexecPrepared(m_Conn,
 	statement.c_str(),
 	nparams,
 	params,
 	paramlengths,
 	binary.c_ptr(),
-	0));
-#else
-  stringstream Q;
-  if (supports(cap_prepared_statements))
-  {
+	0),
+	protocol_version(),
+	statement);
+
+      check_result(r);
+      get_notifs();
+      return r;
+    }
+#endif
+    stringstream Q;
     Q << "EXECUTE \"" << statement << '"';
     if (nparams)
     {
@@ -899,9 +945,11 @@ pqxx::result pqxx::connection_base::prepared_exec(
       }
       Q << ')';
     }
+    r = Exec(Q.str().c_str(), 0);
   }
   else
   {
+    stringstream Q;
     // This backend doesn't support prepared statements.  Do our own variable
     // substitution.
     string S = s.definition;
@@ -916,10 +964,8 @@ pqxx::result pqxx::connection_base::prepared_exec(
 	S.replace(h,keysz,val);
     }
     Q << S;
+    r = Exec(Q.str().c_str(), 0);
   }
-  result r(Exec(Q.str().c_str(), 0));
-#endif
-  check_result(r, statement.c_str());
   get_notifs();
   return r;
 }
@@ -1036,7 +1082,8 @@ void pqxx::connection_base::MakeEmpty(pqxx::result &R)
   if (!m_Conn)
     throw internal_error("MakeEmpty() on null connection");
 
-  R = result(PQmakeEmptyPGresult(m_Conn, PGRES_EMPTY_QUERY));
+  R = result(PQmakeEmptyPGresult(m_Conn, PGRES_EMPTY_QUERY),
+             protocol_version());
 }
 
 
@@ -1058,14 +1105,18 @@ bool pqxx::connection_base::ReadCopyLine(PGSTD::string &Line)
 
 #ifdef PQXX_HAVE_PQPUTCOPY
   char *Buf = 0;
+  const int proto = protocol_version();
+  const string querydesc = "[END COPY]";
   switch (PQgetCopyData(m_Conn, &Buf, false))
   {
     case -2:
       throw runtime_error("Reading of table data failed: " + string(ErrMsg()));
 
     case -1:
-      for (result R(PQgetResult(m_Conn)); R; R=PQgetResult(m_Conn))
-	check_result(R, "[END COPY]");
+      for (result R(PQgetResult(m_Conn), proto, querydesc);
+           R;
+	   R=result(PQgetResult(m_Conn), proto, querydesc))
+	check_result(R);
       Result = false;
       break;
 
@@ -1166,8 +1217,8 @@ void pqxx::connection_base::EndCopyWrite()
 	"from PQputCopyEnd()");
   }
 
-  const result R(PQgetResult(m_Conn));
-  check_result(R, "[END COPY]");
+  const result R(PQgetResult(m_Conn), protocol_version(), "[END COPY]");
+  check_result(R);
 
 #else
   WriteCopyLine(theWriteTerminator);
@@ -1240,7 +1291,8 @@ PGSTD::string pqxx::connection_base::esc_raw(const unsigned char str[],
     throw runtime_error(ErrMsg());
   }
 #else
-  PQAlloc<unsigned char> buf( PQescapeBytea(str, len, &bytes) );
+  unsigned char *const s = const_cast<unsigned char *>(str);
+  PQAlloc<unsigned char> buf( PQescapeBytea(s, len, &bytes) );
   if (!buf.c_ptr()) throw bad_alloc();
 #endif
   return string(reinterpret_cast<char *>(buf.c_ptr()));
@@ -1332,16 +1384,39 @@ int pqxx::connection_base::await_notification(long seconds, long microseconds)
 
 void pqxx::connection_base::read_capabilities() throw ()
 {
-  int v = 0;
 #ifdef PQXX_HAVE_PQSERVERVERSION
-  if (m_Conn) v = PQserverVersion(m_Conn);
+  m_serverversion = PQserverVersion(m_Conn);
+#else
+  m_serverversion = 0;
+  try
+  {
+    // Estimate server version by querying 'version()'.  This may not be exact!
+    const string VQ = "SELECT version()";
+    const result r(PQexec(m_Conn, VQ.c_str()), protocol_version(), VQ);
+    int x=0, y=0, z=0;
+    if ((sscanf(r[0][0].c_str(), "PostgreSQL %d.%d.%d", &x, &y, &z) == 3) &&
+        (x >= 0) && (x < 100) &&
+	(y >= 0) && (y < 100) &&
+	(z >= 0) && (z < 100))
+      m_serverversion = 10000*x + 100*y + z;
+  }
+  catch (const exception &)
+  {
+  }
 #endif
 
+  const int v = m_serverversion;
+
   m_caps[cap_prepared_statements] = (v >= 70300);
   m_caps[cap_cursor_scroll] = (v >= 70400);
   m_caps[cap_cursor_with_hold] = (v >= 70400);
   m_caps[cap_nested_transactions] = (v >= 80000);
   m_caps[cap_create_table_with_oids] = (v >= 80000);
+
+#ifdef PQXX_HAVE_PQFTABLECOL
+  const int p = protocol_version();
+  m_caps[cap_table_column] = (p >= 3);
+#endif
 }
 
 
