@@ -108,6 +108,7 @@ void pqxx::cursor_base::declare(const PGSTD::string &query,
 
   cq << "DECLARE \"" << name() << "\" ";
 
+  m_context->conn().activate();
   if (m_context->conn().supports(connection_base::cap_cursor_scroll))
   {
     if (ap == forward_only) cq << "NO ";
@@ -159,7 +160,13 @@ void pqxx::cursor_base::close() throw ()
 {
   if (m_ownership==owned)
   {
-    try { m_context->exec("CLOSE " + name()); } catch (const exception &) { }
+    try
+    {
+      m_context->exec("CLOSE \"" + name() + "\"");
+    }
+    catch (const exception &)
+    {
+    }
 
     if (m_adopted) m_context->m_reactivation_avoidance.add(-1);
     m_ownership = loose;
