@@ -48,8 +48,9 @@ template<typename ITER> string subst(transaction_base &t,
 	ITER patend)
 {
   int i = distance(patbegin, patend);
-  for (ITER arg = --patend; i > 0; --arg, --i)
+  for (ITER arg = patend; i > 0; --i)
   {
+   --arg;
     const string marker = "$" + to_string(i),
 	  var = stringize(t, *arg);
     const string::size_type msz = marker.size();
@@ -76,23 +77,47 @@ int main()
 {
   try
   {
+    /* A bit of nastiness in prepared statements: on 7.3.x backends we can't
+     * compare pg_tables.tablename to a string.  We work around this by using
+     * the LIKE operator.
+     *
+     * Later backend versions do not suffer from this problem.
+     */
     const string QN_readpgtables = "ReadPGTables",
 	  Q_readpgtables = "SELECT * FROM pg_tables",
 	  QN_seetable = "SeeTable",
-	  Q_seetable = Q_readpgtables + " WHERE tablename=$1",
+	  Q_seetable = Q_readpgtables + " WHERE tablename LIKE $1",
 	  QN_seetables = "SeeTables",
-	  Q_seetables = Q_seetable + " OR tablename=$2";
+	  Q_seetables = Q_seetable + " OR tablename LIKE $2";
 
     lazyconnection C;
+
     cout << "Preparing a simple statement..." << endl;
     C.prepare(QN_readpgtables, Q_readpgtables);
     nontransaction T(C, "test85");
 
-    // See if a basic prepared statement runs consistently with a regular query
-    cout << "Basic correctness check on prepared statement..." << endl;
-    compare_results(QN_readpgtables,
+    try
+    {
+      // See if a basic prepared statement works just like a regular query
+      cout << "Basic correctness check on prepared statement..." << endl;
+      compare_results(QN_readpgtables,
 	T.prepared(QN_readpgtables).exec(),
 	T.exec(Q_readpgtables));
+    }
+    catch (const exception &)
+    {
+      if (!C.supports(connection_base::cap_prepared_statements))
+      {
+        cout << "Backend version does not support prepared statements.  "
+	        "Skipping."
+	     << endl;
+	return 0;
+      }
+      throw;
+    }
+
+    // Try prepare_now() on an already prepared statement
+    C.prepare_now(QN_readpgtables);
 
     // Pro forma check: same thing but with name passed as C-style string
     compare_results(QN_readpgtables+"_char",
@@ -102,6 +127,21 @@ int main()
     cout << "Dropping prepared statement..." << endl;
     C.unprepare(QN_readpgtables);
 
+    bool failed = true;
+    try
+    {
+      disable_noticer d(C);
+      C.prepare_now(QN_readpgtables);
+      failed = false;
+    }
+    catch (const exception &e)
+    {
+      cout << "(Expected) " << e.what() << endl;
+    }
+    if (!failed)
+      throw runtime_error("prepare_now() succeeded on dropped statement");
+
+
     // Just to try and confuse things, "unprepare" twice
     cout << "Testing error detection and handling..." << endl;
     try { C.unprepare(QN_readpgtables); }
@@ -115,6 +155,7 @@ int main()
 
     // Re-prepare the same statement and test again
     C.prepare(QN_readpgtables, Q_readpgtables);
+    C.prepare_now(QN_readpgtables);
     compare_results(QN_readpgtables+"_2",
 	T.prepared(QN_readpgtables).exec(),
 	T.exec(Q_readpgtables));
@@ -207,6 +248,12 @@ int main()
 
     cout << "Done." << endl;
   }
+  catch (const feature_not_supported &e)
+  {
+    cout << "Backend version does not support prepared statements.  Skipping."
+         << endl;
+    return 0;
+  }
   catch (const sql_error &e)
   {
     cerr << "SQL error: " << e.what() << endl
